<?php
 /**
 * Chargement d'un fichier CSV dans une table
 * @param string $source Nom du fichier CSV
 * @param string $table Nom de la table
 * @param string $separateur Separateur de champs
 * @param int $effacement Si 1 alors effacement des donnees sinon pas d'effacement
 * @param $id_sql int Identifiant de connexion
 */
 Function charge_fichier_csv ($source, $table, $separateur, $effacement, $id_sql)
  {
   Global $id_sql ;
   If ($effacement == 1) { $ok = SQL_commande ("DELETE FROM ".$table, $id_sql) ; }
   $ok = SQL_commande ("LOAD DATA LOCAL INFILE '".$source."' INTO TABLE ".$table." FIELDS TERMINATED BY '".$separateur."' ENCLOSED BY '\"' ESCAPED BY '\\\\' LINES TERMINATED BY '\\n'", $id_sql) ;
  }
 
 /**
 * Determine le passage a la ligne en fonction de l'OS
 */
 Function determine_passage_ligne ()
  {
   $passage = "\n" ;
   If (!IsSet ($GLOBALS ['HTTP_USER_AGENT']))
    {
     If (!Empty ($GLOBALS ['HTTP_SERVER_VARS']) && IsSet ($GLOBALS ['HTTP_SERVER_VARS']['HTTP_USER_AGENT']))
      { $GLOBALS ['HTTP_USER_AGENT'] = $GLOBALS ['HTTP_SERVER_VARS']['HTTP_USER_AGENT'] ; }
     Else
      { $GLOBALS ['HTTP_USER_AGENT'] = @GetEnv ('HTTP_USER_AGENT') ; }
    }
   // window$ ?
   If (!Empty ($GLOBALS ['HTTP_USER_AGENT']) && EReg ('[^(]*\((.*)\)[^)]*', $GLOBALS ['HTTP_USER_AGENT'], $regs))
    {
     If (ERegi ('Win', $regs [1])) { $passage = "\r\n" ; }
    }
   Return $passage ;
  }

 /**
 * Obtient le SQL de creation d'une table
 * @param $nom_base string Nom de la base de donnees
 * @param $nom_table string Nom de la table
 * @param $id_sql int Identifiant de connexion
 * @return string Chaine de creation de la table
 */
 Function definition_table ($nom_base, $nom_table, $id_sql)
  {
   $resultat = SQL_commande ("SHOW CREATE TABLE ".$nom_base.".".$nom_table, $id_sql) ;
   If (SQL_nbr_lignes ($resultat) == 1)
    {
     $crlf = determine_passage_ligne () ;
     $r = SQL_ligne_suivante ($resultat) ;
     $definition = Str_Replace ("\n", $crlf, $r [1]).";" ;
     Return $definition ;
    }
   Else { Return "" ; }
  }

 /**
 * Verification et recuperation des notes a partir d'une sauvegarde
 * @param string $nb Nom de la base de donnees
 * @param array $prof Tableau contenant la liste des enseignants a verifier ($prof [0]=nbr si nbr=0 alors tous sinon $prof [x]=id_prof)
 * @param string $nom_fichier_notes Nom du fichier de sauvegarde des notes (CSV)
 * @param string $separateur Separateur de champs pour les CSV
 * @param string $millesime Millesime a traiter
 * @param int $id_sql Identifiant de connexion
 * @param int $id_classe Identifiant de la classe
 */
 Function recup_notes ($nb, $prof, $nom_fichier_notes, $separateur, $millesime, $id_sql, $id_classe)
  {
   // Creation de la table appreciations temporaire
   //$def = definition_table ($nb, "appreciations", $id_sql) ;
   //$def = Str_Replace ("CREATE TABLE `appreciations`", "CREATE TABLE appreciations_temp", $def) ;
   //$ok = SQL_commande ("DROP TABLE appreciations_temp", $id_sql) ;
   //$ok = SQL_commande ($def, $id_sql) ;
   // Creation de la table notes temporaire
   $def = definition_table ($nb, "notes", $id_sql) ;
   $def = Str_Replace ("CREATE TABLE `notes`", "CREATE TABLE notes_temp", $def) ;
   $ok = SQL_commande ("DROP TABLE notes_temp", $id_sql) ;
   $ok = SQL_commande ($def, $id_sql) ;
   
   // Aucune limite de traitement
   @Set_Time_Limit (0) ;
   
   // Remplissage tables temporaires
   //charge_fichier_csv ($nom_fichier_app, "appreciations_temp", $separateur, 1, $id_sql) ;
   charge_fichier_csv ($nom_fichier_notes, "notes_temp", $separateur, 1, $id_sql) ;
   // Recherche de tous les controles
   $controles = SQL_commande ("SELECT id_controle FROM controles WHERE classe='".$id_classe."'", $id_sql) ;
   $nb_c = SQL_nbr_lignes ($controles) ;
   For ($i = 0 ; $i < $nb_c ; $i++)
    {
     $controle = SQL_ligne_suivante ($controles) ;
     // Recherche les changements
     $resultats = SQL_commande ("SELECT n.id_note,p.valeur AS valeur_temp FROM notes n,notes_temp p WHERE n.eleve=p.eleve AND n.controle='".$controle [0]."' AND n.controle=p.controle AND n.valeur!=p.valeur AND n.millesime=p.millesime AND n.millesime='".$millesime."'", $id_sql) ;
     $nb_r = SQL_nbr_lignes ($resultats) ;
     For ($j = 0 ; $j < $nb_r ; $j++)
      {
       $res = SQL_ligne_suivante ($resultats) ;
       $ok = SQL_commande ("UPDATE notes SET valeur='".$res [1]."' WHERE id_note='".$res [0]."' LIMIT 1", $id_sql) ;
      }
    }
   // Fin de traitement : Destruction des tables
   $ok = SQL_commande ("DROP TABLE notes_temp", $id_sql) ;
   Return 1 ;
  }

 /**
 * Function de generation de la structure de la base de donnees en SQL
 * @param array $table Noms des tables a sauvegarder [0]=Nombre;[x>1]=Nom de la table
 * @return string Structure en SQL
 */
 Function structure_table_SQL ($table)
  {
   Global $id_sql, $base_choisie, $version_garennes ;
   $pl = determine_passage_ligne () ;
   $chaine  = "# =================================================================================".$pl ;
   $chaine .= "#  Structure de la base de donnees : ".$base_choisie.$pl ;
   $chaine .= "#  Version : ".$version_garennes.$pl ;
   $chaine .= "#  Date de creation : ".Date ("d-m-Y").$pl ;
   $chaine .= "#  Heure de creation : ".Date ("H:i:s").$pl ;
   $chaine .= "# -- (c)JFC ".Date ("Y")." - GNU GPL --".$pl ;
   $chaine .= "# =================================================================================".$pl ;
   For ($i = 1 ; $i <= $table [0] ; $i++)
    {
     $res = SQL_commande ("SHOW CREATE TABLE ".$table [$i], $id_sql) ;
     $r = SQL_ligne_suivante ($res) ;
     //$chaine .= "# =====---__ TABLE n".$i." : ".$table [$i]."__---=====".$pl ;
     $chaine .= "DROP TABLE IF EXISTS ".$table [$i].";".$pl ;
     $chaine .= Str_Replace ("  ", "", Str_Replace ("\n", "", $r [1])).";".$pl ;
    }
   Return $chaine ;
  }
 
 /**
 * Function de generation des donnees de la base en SQL
 * @param array $table Noms des tables a sauvegarder [0]=Nombre;[x>1]=Nom de la table
 * @return string Donnees en SQL
 */
 Function donnees_table_SQL ($table)
  {
   Global $id_sql, $base_choisie, $version_garennes ;
   $pl = determine_passage_ligne () ;
   $chaine  = "# =================================================================================".$pl ;
   $chaine .= "#  Donnees de la base de donnees : ".$base_choisie.$pl ;
   $chaine .= "#  Version : ".$version_garennes.$pl ;
   $chaine .= "#  Date de creation : ".Date ("d-m-Y").$pl ;
   $chaine .= "#  Heure de creation : ".Date ("H:i:s").$pl ;
   $chaine .= "# -- (c)JFC ".Date ("Y")." - GNU GPL --".$pl ;
   $chaine .= "# =================================================================================".$pl ;
   For ($i = 1 ; $i <= $table [0] ; $i++)
    {
     $res = SQL_commande ("SELECT * FROM ".$table [$i], $id_sql) ;
     $nbr = SQL_nbr_lignes ($res) ;
     $nbr_champs = SQL_nbr_champs ($res) ;
     //$chaine .= "# =====---__ TABLE n".$i." : ".$table [$i]." __---=====".$pl ;
     If ($nbr > 0)
      {
       $entete = "INSERT INTO `".$table [$i]."` VALUES " ;
       $chaine_tempo = $entete ;
       $compteur = 0 ;
       For ($j = 0 ; $j < $nbr ; $j++)
        {
         $r = SQL_ligne_suivante ($res) ;
         If (($j > 0) And ($compteur > 0)) { $chaine_tempo .= ",".$pl."(" ; } Else { $chaine_tempo .= "(" ; }
         For ($k = 0 ; $k < $nbr_champs ; $k++)
          {
           If ($k > 0) { $chaine_tempo .= "," ; }
           $chaine_tempo .= "'".Str_Replace ("\n", "\\n", Str_Replace ("'", "''", $r [$k]))."'" ;
          }
         $chaine_tempo .= ")" ;
         $compteur++ ;
         If ($compteur > 100) { $compteur = 0 ; $chaine .= $entete.$chaine_tempo.";".$pl ; $chaine_tempo = "" ; }
        }
       $chaine_tempo .= ";".$pl ;
       If ($compteur > 0) { $chaine .= $chaine_tempo ; }
      }
    }
   Return $chaine ;
  }
?>